library(dplyr)
library(randomForest)
library(caret)

##### Set seed 
set.seed(1011)

## Get features
train_tab = read.table("RFQAmodel_Training.txt", header=TRUE)
val_tab = read.table("RFQAmodel_Validation.txt", header=TRUE)
train_tab <- train_tab %>% mutate(PPV=PPV/100.0, PPV_Max=PPV_Max/100.0, PPV_Num=PPV_Num/100.0)
val_tab <- val_tab %>% mutate(PPV=PPV/100.0, PPV_Max=PPV_Max/100.0, PPV_Num=PPV_Num/100.0)

## Label models as correct or incorrect 
train_tab <- train_tab %>% mutate(Label = ifelse(TMScore >= 0.5, "Correct","Incorrect"))
val_tab <- val_tab %>% mutate(Label = ifelse(TMScore >= 0.5, "Correct","Incorrect"))

# Use Random Forest Classifier to classify models into Correct/Incorrect (or load a pre-trained version)
#RFQAmodel <- randomForest(as.factor(Label) ~ . -Decoy -Target -TMScore -SCOP_Class, data=train_tab, importance=TRUE, ntree=500,mtry = 7)
load("RFQAmodel_classifier.Rda")

## Classify the Validation set
prediction = predict(RFQAmodel,val_tab,type="prob")
results <- cbind(val_tab, prediction)

## Categorise predictions into confidence levels
results2 <- results %>% group_by(Target) %>% mutate(Confidence = ifelse(max(Correct) > 0.5 , "High", ifelse(max(Correct) > 0.3, "Medium", ifelse(max(Correct) > 0.1, "Low","Failed"))))

## Add "All", "High and Medium", and "Predicted Modelling", and get the number modelling successes (Max), number of correct highest-ranking (top1), best of five highest ranking (top5), the totalnumber of targets (total), as well as precision, for each confidence level
stats <- results2 %>% bind_rows((results2 %>% mutate(Confidence = "All")), (results2 %>% filter(Confidence %in% c("High","Medium")) %>% mutate(Confidence = "High.and.Medium")), results2 %>% filter(Confidence != "Failed") %>% mutate(Confidence = "Predicted.Modelling.Success")) %>% group_by(Confidence, Target) %>% mutate(Best=max(TMScore)>=0.5) %>% arrange(-Correct) %>% slice(1:5) %>%  mutate(Top5=max(TMScore >=0.5)) %>% slice(1) %>% summarise(Top5=Top5, Top1=sum(TMScore>=0.5), Max=sum(Best)) %>% summarise(Top5=sum(Top5), Top1=sum(Top1),Max=sum(Max),Total=length(Confidence)) %>% mutate(Precision.Top5=Top5/Total, Precision.Top1=Top1/Total)

print(stats)

## CASP12 Test set
test_tab <- read.table("RFQAmodel_CASP12Test.txt", header=TRUE)
test_tab <- test_tab %>% mutate(PPV=PPV/100.0, PPV_Max=PPV_Max/100.0, PPV_Num=PPV_Num/100.0)

## Classify models
CASP12_prediction <- predict(RFQAmodel, test_tab,type="prob")
CASP12_results <- cbind(test_tab, CASP12_prediction)

## Categorise predictions
CASP12_results2 <- CASP12_results %>% group_by(Target) %>% mutate(Confidence = ifelse(max(Correct) > 0.5 , "High", ifelse(max(Correct) > 0.3, "Medium", ifelse(max(Correct) > 0.1, "Low","Failed"))))

## Calculate precision
CASP12_stats <- CASP12_results2 %>% bind_rows((CASP12_results2 %>% mutate(Confidence = "All")), (CASP12_results2 %>% filter(Confidence %in% c("High","Medium")) %>% mutate(Confidence = "High.and.Medium")), CASP12_results2 %>% filter(Confidence != "Failed") %>% mutate(Confidence = "Predicted.Modelling.Success")) %>% group_by(Confidence, Target) %>% mutate(Best=max(TMScore)>=0.5) %>% arrange(-Correct) %>% slice(1:5) %>%  mutate(Top5=max(TMScore >=0.5)) %>% slice(1) %>% summarise(Top5=Top5, Top1=sum(TMScore>=0.5), Max=sum(Best)) %>% summarise(Top5=sum(Top5), Top1=sum(Top1),Max=sum(Max),Total=length(Confidence)) %>% mutate(Precision.Top5=Top5/Total, Precision.Top1=Top1/Total)

print(CASP12_stats)
